-----------------------------------------------------------------------------------------------
-- Client Lua Script for Revolver
-- Copyright (c) NCsoft. All rights reserved
-- Sprites created and code adapted by Dularu
-----------------------------------------------------------------------------------------------

require "Window"


local Revolver = {}

local floor = math.floor

function Revolver:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

  -- Default settings
	self.revlocation = { 1000, -820, 1600, -222 }
	self.butlocation = { 1120, 530, 1520, 630 }
	self.scale = 0.70
	self.butscale = 0.59
	self.lock = false
	self.HFocus1 = true
	self.HFocus2 = true
	self.HFocus3 = true
	self.HFocus4 = true
	self.combattoggle = false
	self.CHUD1 = false
	self.CHUD2 = false
	self.ItemBar = true
	self.ItemBarBG = true
		
    return o
end

function Revolver:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Revolver"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- Setup
-----------------------------------------------------------------------------------------------
function Revolver:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("Revolver.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self)
	Apollo.LoadSprites("RevSprites.xml")
end

function Revolver:OnDocumentReady()
	if self.xmlDoc == nil then
		return
	end

	self.bDocLoaded = true
	self:OnRequiredFlagsChanged()
end

function Revolver:OnRequiredFlagsChanged()
	if g_wndActionBarResources and self.bDocLoaded then
		if GameLib.GetPlayerUnit() then
			self:OnCharacterCreated()
		else
			Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
		end
	end
end

function Revolver:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end

	local eClassId =  unitPlayer:GetClassId()
	if eClassId == GameLib.CodeEnumClass.Spellslinger then
		self:OnCreateSlinger()
	elseif unitPlayer:GetClassId() == GameLib.CodeEnumClass.Esper then
		self:NotSlinger()
	elseif unitPlayer:GetClassId() == GameLib.CodeEnumClass.Engineer then
		self:NotSlinger()
	elseif unitPlayer:GetClassId() == GameLib.CodeEnumClass.Medic then
		self:NotSlinger()
	elseif unitPlayer:GetClassId() == GameLib.CodeEnumClass.Warrior then
		self:NotSlinger()
	elseif unitPlayer:GetClassId() == GameLib.CodeEnumClass.Stalker then
		self:NotSlinger()
	end
end

function Revolver:OnSave(saveDepth)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end
	
	local savedVariables = {}
	savedVariables.scale = self.scale
	savedVariables.butscale = self.butscale
	savedVariables.lock = self.lock
	savedVariables.HFocus1 = self.HFocus1
	savedVariables.HFocus2 = self.HFocus2
	savedVariables.HFocus3 = self.HFocus3
	savedVariables.HFocus4 = self.HFocus4
	savedVariables.combattoggle = self.combattoggle
	savedVariables.CHUD1 = self.CHUD1
	savedVariables.CHUD2 = self.CHUD2
	savedVariables.ItemBar = self.ItemBar
	savedVariables.ItemBarBG = self.ItemBarBG
	
	local s, h, i, t = self.wndMain:GetAnchorOffsets()
	savedVariables.revlocation = { s, h, i, t }
	local c, r, a, p = self.wndButtons:GetAnchorOffsets()
	savedVariables.butlocation = { c, r, a, p }
	
	return savedVariables
end

function Revolver:OnRestore(saveDepth, savedVariables)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if savedVariables == {} or savedVariables == nil then return end
	
	if savedVariables.scale ~= nil then self.scale = savedVariables.scale end
	if savedVariables.butscale ~= nil then self.butscale = savedVariables.butscale end
	if savedVariables.lock ~= nil then self.lock = savedVariables.lock end
	if savedVariables.HFocus1 ~= nil then self.HFocus1 = savedVariables.HFocus1 end
	if savedVariables.HFocus2 ~= nil then self.HFocus2 = savedVariables.HFocus2 end
	if savedVariables.HFocus3 ~= nil then self.HFocus3 = savedVariables.HFocus3 end
	if savedVariables.HFocus4 ~= nil then self.HFocus4 = savedVariables.HFocus4 end
	if savedVariables.combattoggle ~= nil then self.combattoggle = savedVariables.combattoggle end
	if savedVariables.CHUD1 ~= nil then self.CHUD1 = savedVariables.CHUD1 end
	if savedVariables.CHUD2 ~= nil then self.CHUD2 = savedVariables.CHUD2 end
	if savedVariables.ItemBar ~= nil then self.ItemBar = savedVariables.ItemBar end
	if savedVariables.ItemBarBG ~= nil then self.ItemBarBG = savedVariables.ItemBarBG end
	if savedVariables.revlocation ~= nil then self.revlocation = savedVariables.revlocation end
	if savedVariables.butlocation ~= nil then self.butlocation = savedVariables.butlocation end
end

-----------------------------------------------------------------------------------------------
-- Main Functions
-----------------------------------------------------------------------------------------------
function Revolver:OnCreateSlinger()
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnSlingerUpdateTimer", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnSlingerEnteredCombat", self)
	Apollo.RegisterEventHandler("ActionBarLoaded", "OnRequiredFlagsChanged", self)
	Apollo.RegisterEventHandler("ShowActionBarShortcut", "OnShowActionBarShortcut", self)
	Apollo.RegisterEventHandler("AbilityBookChange", "OnAbilityBookChange",self)
	Apollo.RegisterEventHandler("ToggleAbilitiesWindow", "OnAbilityBookChange", self)
	Apollo.RegisterEventHandler("CharacterEldanAugmentationsUpdated", "OnAbilityBookChange", self)
	Apollo.RegisterEventHandler("SpecChanged", "OnAbilityBookChange", self)
	Apollo.RegisterEventHandler("OptionsUpdated_HUDPreferences", "InitializeControls", self)
	Apollo.RegisterSlashCommand("revolver", "OnConfigure", self)
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "RevolverMain", g_wndActionBarResources, self)
	self.wndMain:FindChild("LargeSurgeSigil"):Show(false, true)
	self.wndMain:FindChild("SlingerBaseFrame_InCombat"):Show(false, true)
	self.wndMain:ToFront()
	self.wndButtons = Apollo.LoadForm(self.xmlDoc, "RevolverButtons", nil, self)
	self.wndForm = Apollo.LoadForm(self.xmlDoc, "RevolverForm", nil, self)
	self.wndForm:Show(false, true)
		
	local nLeft, nTop, nRight, nBottom = self.wndMain:GetRect()
	Apollo.SetGlobalAnchor("CastingBarBottom", 0.0, nTop, true)
	
	self.wndMain:FindChild("BSpell1"):SetScale(1.32)
	self.wndMain:FindChild("BSpell2"):SetScale(1.32)
	self.wndMain:FindChild("BSpell3"):SetScale(1.32)
	self.wndMain:FindChild("BSpell4"):SetScale(1.32)
	self.wndMain:FindChild("BSpell5"):SetScale(1.32)
	self.wndMain:FindChild("BSpell6"):SetScale(1.32)
	self.wndMain:FindChild("BSpell7"):SetScale(1.32)
	self.wndMain:FindChild("BSpell8"):SetScale(1.32)
	self.wndMain:FindChild("VSpell1"):SetScale(1.32)
	self.wndMain:FindChild("VSpell2"):SetScale(1.32)
	self.wndMain:FindChild("VSpell3"):SetScale(1.32)
	self.wndMain:FindChild("VSpell4"):SetScale(1.32)
	self.wndMain:FindChild("VSpell5"):SetScale(1.32)
	self.wndMain:FindChild("VSpell6"):SetScale(1.32)
	self.wndMain:FindChild("VSpell7"):SetScale(1.32)
	self.wndMain:FindChild("VSpell8"):SetScale(1.32)
	
	self.wndForm:FindChild("Part3"):SetOpacity(0.6)
	self.wndForm:FindChild("Part7"):SetOpacity(0.4)
	self.wndButtons:FindChild("Gloss2"):SetOpacity(0.30)
	self.wndButtons:FindChild("NextPot"):SetOpacity(0.9)
	
  -- Apply saved settings
	self.wndMain:SetAnchorOffsets(unpack(self.revlocation))
	self.wndButtons:SetAnchorOffsets(unpack(self.butlocation))
	self.wndMain:SetScale(self.scale)
	self.wndButtons:SetScale(self.butscale)
	self.wndForm:FindChild("Slider"):SetValue(self.scale)
	self.wndForm:FindChild("SliderLabel"):SetText(string.format("Scale (%.2f):", self.scale))
	self.wndForm:FindChild("ButSlider"):SetValue(self.butscale)
	self.wndForm:FindChild("ButSliderLabel"):SetText(string.format("Scale (%.2f):", self.butscale))
	self.wndForm:FindChild("Lock"):FindChild("LockBtn"):SetCheck(self.lock)
	self:OnLockToggle()
	self.wndForm:FindChild("Combat"):FindChild("CombatBtn"):SetCheck(self.combattoggle)
	self.wndForm:FindChild("HFocus"):FindChild("HFocusBtn1"):SetCheck(self.HFocus1)
	self.wndForm:FindChild("HFocus"):FindChild("HFocusBtn2"):SetCheck(self.HFocus2)
	self.wndForm:FindChild("HFocus"):FindChild("HFocusBtn3"):SetCheck(self.HFocus3)
	self.wndForm:FindChild("HFocus"):FindChild("HFocusBtn4"):SetCheck(self.HFocus4)
	self:OnHideFocus()
	self.wndForm:FindChild("CarbHUD"):FindChild("CarbHUDBtn1"):SetCheck(self.CHUD1)
	self.wndForm:FindChild("CarbHUD"):FindChild("CarbHUDBtn2"):SetCheck(self.CHUD2)
	self:CHUD()
	self.wndForm:FindChild("IBar"):FindChild("IBarBtn"):SetCheck(self.ItemBar)
	self:ItemBarControl()
	self.wndForm:FindChild("IBarBG"):FindChild("IBarBGBtn"):SetCheck(self.ItemBarBG)
	self:ItemBarBGControl()
	
	self:OnAbilityCheck()
	self:OnFactionCheck()
	
	self.nFadeLevel = 0
	self.xmlDoc = nil

	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer then
		self:OnSlingerEnteredCombat(unitPlayer, unitPlayer:IsInCombat())
	end
end

function Revolver:OnSlingerUpdateTimer()
	local unitPlayer = GameLib.GetPlayerUnit()
	local nResourceCurrent = unitPlayer:GetResource(4)
	local nResourceMax = unitPlayer:GetMaxResource(4)
	local bSurgeActive = GameLib.IsSpellSurgeActive()
	local bInCombat = unitPlayer:IsInCombat()
	local nEvadeCurr = unitPlayer:GetResource(7)
	local nEvadeMax = unitPlayer:GetMaxResource(7)

  -- Mana
	local nManaMax = math.floor(unitPlayer:GetMaxMana())
	local nManaCurrent = math.floor(unitPlayer:GetMana())
	self.wndMain:FindChild("ManaProgressBar"):SetMax(nManaMax)
	self.wndMain:FindChild("ManaProgressBar"):SetProgress(nManaCurrent)
	self.wndMain:FindChild("ManaProgressBar"):SetTooltip(String_GetWeaselString(Apollo.GetString("SpellslingerResource_FocusTooltip"), nManaCurrent, nManaMax))
	self.wndMain:FindChild("ManaProgressBar"):SetStyleEx("EdgeGlow", bInCombat and (nManaCurrent / nManaMax < 0.97))
	self.wndMain:FindChild("ManaProgressText"):SetText(nManaCurrent == nManaMax and "" or (math.floor(nManaCurrent / nManaMax * 100).."%"))
	self.wndMain:FindChild("ManaProgressText"):SetTextColor(bInCombat and ApolloColor.new("ffffc757") or ApolloColor.new("UI_TextHoloTitle"))
	self.wndMain:FindChild("ManaProgressBacker"):Show(nManaCurrent ~= nManaMax)

  -- Surge
	local bThisBubbleFilled, nodeProgress = nResourceCurrent >= nResourceMax, self.wndMain:FindChild("NodeProgress")
	nodeProgress:SetMax(nResourceMax)
	nodeProgress:SetProgress(nResourceCurrent,nResourceMax) 

	if bThisBubbleFilled then
		self.wndMain:FindChild("NodeFilled"):Show(true, false, 3)
	else
		self.wndMain:FindChild("NodeFilled"):Show(false, false, 3)
	end
	
	if bSurgeActive then
		self.wndMain:FindChild("NodeFilled"):SetSprite("RevSprite:Rev2b")
		self.wndMain:FindChild("NodeProgress"):SetFullSprite("RevSprite:Rev2b")
		self.wndMain:FindChild("ManaProgressBar"):SetFullSprite("RevSprite:Rev7b")
		self.wndMain:FindChild("DashA"):SetSprite("RevSprite:Rev8c")
		self.wndMain:FindChild("DashB"):SetSprite("RevSprite:Rev8d")
		self.wndMain:FindChild("BUL_Sa1"):SetSprite("RevSprite:Bb1")
		self.wndMain:FindChild("BUL_Sa2"):SetSprite("RevSprite:Bb2")
		self.wndMain:FindChild("BUL_Sa3"):SetSprite("RevSprite:Bb3")
		self.wndMain:FindChild("BUL_Sa4"):SetSprite("RevSprite:Bb4")
	else
		self.wndMain:FindChild("NodeFilled"):SetSprite("RevSprite:Rev2")
		self.wndMain:FindChild("NodeProgress"):SetFullSprite("RevSprite:Rev2")
		self.wndMain:FindChild("ManaProgressBar"):SetFullSprite("RevSprite:Rev7")
		self.wndMain:FindChild("DashA"):SetSprite("RevSprite:Rev8a")
		self.wndMain:FindChild("DashB"):SetSprite("RevSprite:Rev8b")
		self.wndMain:FindChild("BUL_Sa1"):SetSprite("RevSprite:Ba1")
		self.wndMain:FindChild("BUL_Sa2"):SetSprite("RevSprite:Ba2")
		self.wndMain:FindChild("BUL_Sa3"):SetSprite("RevSprite:Ba3")
		self.wndMain:FindChild("BUL_Sa4"):SetSprite("RevSprite:Ba4")
	end

	self.wndMain:FindChild("LargeSurgeSigil"):Show(bSurgeActive, bSurgeActive, 0.4)
		
  -- Shots
	if nResourceCurrent >= nResourceMax * 1 then
		self.wndMain:FindChild("BUL_Sa1"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Sa1"):Show(false, false, 3)
	end
	if nResourceCurrent > nResourceMax * 0.725 then
		self.wndMain:FindChild("BUL_Sa2"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Sa2"):Show(false, false, 3)
	end
	if nResourceCurrent >= nResourceMax * 0.50 then
		self.wndMain:FindChild("BUL_Sa3"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Sa3"):Show(false, false, 3)
	end
	if nResourceCurrent > nResourceMax * 0.25 then
		self.wndMain:FindChild("BUL_Sa4"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Sa4"):Show(false, false, 3)
	end
		
	-- Dash	
	if nEvadeCurr >= nEvadeMax / 2 then
		self.wndMain:FindChild("DashA"):Show(true, false, 3)
	else
		self.wndMain:FindChild("DashA"):Show(false, false, 3)
	end
	if nEvadeCurr >= nEvadeMax / 1 then
		self.wndMain:FindChild("DashB"):Show(true, false, 3)
	else
		self.wndMain:FindChild("DashB"):Show(false, false, 3)
	end
	
	-- Abilities
	if self.splAssassinate then
		self:OnAssassinate()
	end
--	if self.splHTorrent then
--		self:OnHTorrent()
--	end
	
--	self:Ontesttest()
end

function Revolver:OnSlingerEnteredCombat(unitPlayer, bInCombat)
	if unitPlayer ~= GameLib.GetPlayerUnit() or not self.wndMain or not self.wndMain:IsValid() then
		return
	end

	if bInCombat then
		self.wndMain:FindChild("SlingerBaseFrame_InCombat"):Show(true, false, 3)
		self.wndMain:FindChild("LargeSurgeSigil"):SetSprite("RevSprite:Rev5")
		self.wndMain:FindChild("DashBG_InCombat"):Show(true, false, 3)
	else
		self.wndMain:FindChild("SlingerBaseFrame_InCombat"):Show(false, false, 3)
		self.wndMain:FindChild("LargeSurgeSigil"):SetSprite("RevSprite:Rev5")
		self.wndMain:FindChild("DashBG_InCombat"):Show(false, false, 3)
	end

	if self.combattoggle == true and bInCombat then
		self.wndMain:Show(true, false, 3)
	elseif self.combattoggle == true and not bInCombat then
		self.wndMain:Show(false, false, 3)
	end
end

function Revolver:OnGenerateTooltip(wndControl, wndHandler, eType, arg1, arg2)
	local xml = nil
	if eType == Tooltip.TooltipGenerateType_ItemInstance then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_ItemData then -- Doesn't need to compare to item equipped
		Tooltip.GetItemTooltipForm(self, wndControl, arg1, {})
	elseif eType == Tooltip.TooltipGenerateType_GameCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Macro then
		xml = XmlDoc.new()
		xml:AddLine(arg1)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Spell then
		if Tooltip ~= nil and Tooltip.GetSpellTooltipForm ~= nil then
			Tooltip.GetSpellTooltipForm(self, wndControl, arg1)
		end
	elseif eType == Tooltip.TooltipGenerateType_PetCommand then
		xml = XmlDoc.new()
		xml:AddLine(arg2)
		wndControl:SetTooltipDoc(xml)
	end
end

  -- Vehicle action bar
function Revolver:OnShowActionBarShortcut(eWhichBar, bIsVisible, nNumShortcuts)
	if eWhichBar == ActionSetLib.CodeEnumShortcutSet.VehicleBar and self.wndMain and self.wndMain:IsValid() then
		self:OnShowVehicleBar(eWhichBar, bIsVisible, nNumShortcuts) -- show/hide vehicle bar if eWhichBar matches
	end
end

function Revolver:OnShowVehicleBar(eWhichBar, bIsVisible, nNumShortcuts)
	if eWhichBar ~= ActionSetLib.CodeEnumShortcutSet.VehicleBar or not self.wndMain or not self.wndMain:IsValid() then
		return
	end

	local wndVehicleBar = self.wndMain:FindChild("VehicleSpells")
	wndVehicleBar:Show(bIsVisible)
	
	self.wndMain:FindChild("SpellsMain"):Show(not bIsVisible)

	local unitPlayer = GameLib:GetPlayerUnit()
	if unitPlayer and not unitPlayer:IsInVehicle() then
		self.tCurrentVehicleInfo = nil
	end

	if bIsVisible then
		wndVehicleBar:Show(false)
	end

	if nNumShortcuts then
		wndVehicleBar:Show(true)
	end
end

-----------------------------------------------------------------------------------------------
-- Bullets
-----------------------------------------------------------------------------------------------
function Revolver:OnAssassinate()
	if self.splAssassinate then
	local tAssassinCharges = self.splAssassinate:GetAbilityCharges()
	if (tAssassinCharges.nChargesRemaining > 2) then
		self.wndMain:FindChild("BUL_Aa3"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Aa3"):Show(false, false, 3)
	end
	if (tAssassinCharges.nChargesRemaining > 1) then
		self.wndMain:FindChild("BUL_Aa2"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Aa2"):Show(false, false, 3)
	end
	if (tAssassinCharges.nChargesRemaining > 0) then
		self.wndMain:FindChild("BUL_Aa1"):Show(true, false, 3)
	else
		self.wndMain:FindChild("BUL_Aa1"):Show(false, false, 3)
	end
	
	-- below 30% health
	local unitPlayer = GameLib.GetPlayerUnit()
	local unitTarget = nil 
	if unitPlayer ~= nil then
		unitTarget = unitPlayer:GetTarget()
	end

	if not unitTarget then
		self.wndMain:FindChild("BUL_Aa1"):SetSprite("RevSprites:Bd1")
		self.wndMain:FindChild("BUL_Aa2"):SetSprite("RevSprites:Bd2")
		self.wndMain:FindChild("BUL_Aa3"):SetSprite("RevSprites:Bd3")
		return
	end
	
	local nMaxHealth = unitTarget:GetMaxHealth()
	local nHealth = unitTarget:GetHealth() 
	if nMaxHealth == nil or nMaxHealth == 0 or nHealth == nil then
		self.wndMain:FindChild("BUL_Aa1"):SetSprite("RevSprites:Bd1")
		self.wndMain:FindChild("BUL_Aa2"):SetSprite("RevSprites:Bd2")
		self.wndMain:FindChild("BUL_Aa3"):SetSprite("RevSprites:Bd3")
		return
	end
	
	local nHealthInPerc = floor((nHealth / nMaxHealth * 100) + 0.5) 
	if (nHealthInPerc > 30) or (nHealthInPerc == 0) then
		self.wndMain:FindChild("BUL_Aa1"):SetSprite("RevSprites:Bd1") 
		self.wndMain:FindChild("BUL_Aa2"):SetSprite("RevSprites:Bd2")
		self.wndMain:FindChild("BUL_Aa3"):SetSprite("RevSprites:Bd3")
	elseif (nHealthInPerc < 30) then 
		self.wndMain:FindChild("BUL_Aa1"):SetSprite("RevSprites:Be1")
		self.wndMain:FindChild("BUL_Aa2"):SetSprite("RevSprites:Be2")
		self.wndMain:FindChild("BUL_Aa3"):SetSprite("RevSprites:Be3")
	end
	end
end

--function Revolver:OnHTorrent()
--	if self.splHTorrent then
--	local tHTorrentCooldown = self.splHTorrent:GetCooldownRemaining()
--	if tHTorrentCooldown > 0 then  --OnCooldown
--		self.wndMain:FindChild("BUL_Ha1"):Show(false, false, 3)
--	elseif tHTorrentCooldown <= 0 then  --OffCooldown
--		self.wndMain:FindChild("BUL_Ha1"):Show(true, false, 3)
--	end
--	end
--end

--function Revolver:Ontesttest()
--    local unitPlayer = GameLib.GetPlayerUnit()
--    local tableBeneficialBuffs = unitPlayer:GetBuffs().arBeneficial
--    for _, buffTable in next, tableBeneficialBuffs do
--        if buffTable.idBuff == 23465 then
--	    	self.wndMain:FindChild("TEST"):Show(false, false, 3)
--        end
--    end
--end

-----------------------------------------------------------------------------------------
-- Potions/Stargate
-----------------------------------------------------------------------------------------------
function Revolver:OnNextPot(wndHandler, wndControl)
	local luaActionBarFrame = Apollo.GetAddon("ActionBarFrame")
		if luaActionBarFrame then
			--luaActionBarFrame.nSelectedPotion = 
			luaActionBarFrame:RedrawPotions()
		end
end



function Revolver:OnFactionCheck()
	self.wndButtons:FindChild("Stargate4"):Show(false)
	self.wndButtons:FindChild("Stargate3"):Show(false)
	local unitPlayer = GameLib.GetPlayerUnit()
	local faction = unitPlayer:GetFaction()
	if faction == Unit.CodeEnumFaction.ExilesPlayer then
		self.wndButtons:FindChild("Stargate4"):Show(true)
    elseif faction == Unit.CodeEnumFaction.DominionPlayer then
		self.wndButtons:FindChild("Stargate3"):Show(true)
    end
end
--------------------------------------------------------------------------------
-- Options
-----------------------------------------------------------------------------------------------
  -- Buttons
function Revolver:OnConfigure()
    self.wndForm:Invoke()
end

function Revolver:OnOK()
	self.wndForm:Close()
end

function Revolver:OnRevert()
	self.lock = false
	self.wndForm:FindChild("Lock"):FindChild("LockBtn"):SetCheck(false)
	self:OnLockToggle()
	self.wndMain:SetScale(0.70)
	self:OnScaleChange()
	self.wndMain:SetAnchorOffsets(1000, -820, 1600, -222)
	self.wndButtons:SetAnchorOffsets(1120, 530, 1520, 630)
end

  -- Book change
function Revolver:OnAbilityBookChange()
	self:OnHideFocus()
	self:OnAbilityCheck()
end

function Revolver:OnAbilityCheck()
	local currentAbilities = ActionSetLib.GetCurrentActionSet()
	local allAbilities = AbilityBook.GetAbilitiesList()
	
	if (allAbilities == nil) then 
		return 
	end

	for index = 1, 10 do
    	currentAbilities[currentAbilities[index]] = true; currentAbilities[index] = nil
	end
	
  -- Assassinate
	if currentAbilities[23274] then
    	for index, abilityTable in next, allAbilities do
        	if abilityTable.nId == 23274 then
				self.splAssassinate = abilityTable.tTiers[abilityTable.nCurrentTier].splObject
        	end
    	end
	elseif not currentAbilities[23274] then
		self.splAssassinate = nil
		self.wndMain:FindChild("BUL_Aa1"):Show(false, false, 3)
		self.wndMain:FindChild("BUL_Aa2"):Show(false, false, 3)
		self.wndMain:FindChild("BUL_Aa3"):Show(false, false, 3)
	end
	
  -- HealingTorrent
--	if currentAbilities[23463] then
--    	for index, abilityTable in next, allAbilities do
--        	if abilityTable.nId == 23463 then
--				self.splHTorrent = abilityTable.tTiers[abilityTable.nCurrentTier].splObject
--        	end
--    	end
--	elseif not currentAbilities[23463] then
--		self.splHTorrent = nil
--		self.wndMain:FindChild("BUL_Ha1"):Show(false, false, 3)
--	end
end

  -- Toggles
function Revolver:OnConfigLockToggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
  	self.lock = wndControl:IsChecked()
	self:OnLockToggle()
end

function Revolver:OnConfigHFocus1Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.HFocus1 = wndControl:IsChecked()
	self:OnHideFocus()
end

function Revolver:OnConfigHFocus2Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.HFocus2 = wndControl:IsChecked()
	self:OnHideFocus()
end

function Revolver:OnConfigHFocus3Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.HFocus3 = wndControl:IsChecked()
	self:OnHideFocus()
end

function Revolver:OnConfigHFocus4Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.HFocus4 = wndControl:IsChecked()
	self:OnHideFocus()
end

function Revolver:OnConfigInCombatToggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.combattoggle = wndControl:IsChecked()
	
	if self.combattoggle == false then
		self.wndMain:Show(true, false, 3)
	elseif self.combattoggle == true then
		self.wndMain:Show(false, false, 3)
	end
end

function Revolver:OnConfigCHUD1Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.CHUD1 = wndControl:IsChecked()
	self:CHUD()
end

function Revolver:OnConfigCHUD2Toggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
	self.CHUD2 = wndControl:IsChecked()
	self:CHUD()
end

function Revolver:OnConfigIBarToggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
  	self.ItemBar = wndControl:IsChecked()
	self:ItemBarControl()
end

function Revolver:OnConfigIBarBGToggle( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then return end
  	self.ItemBarBG = wndControl:IsChecked()
	self:ItemBarBGControl()
end

  -- Option functions
function Revolver:OnLockToggle()
	if  self.lock == true then
		self.wndMain:SetStyle("Moveable", false)
		self.wndMain:SetStyle("IgnoreMouse", true)
		self.wndButtons:SetStyle("Moveable", false)
		self.wndButtons:SetStyle("IgnoreMouse", true)
	elseif self.lock == false then
		self.wndMain:SetStyle("Moveable", true)
		self.wndMain:SetStyle("IgnoreMouse", false)
		self.wndButtons:SetStyle("Moveable", true)
		self.wndButtons:SetStyle("IgnoreMouse", false)
	end
end

function Revolver:OnHideFocus()
	local LASbook = AbilityBook.GetCurrentSpec()
	local FocusWin = self.wndMain:FindChild("ManaContainer")
    if LASbook == 1 and self.HFocus1 == true then
		FocusWin:Show(true, false, 3)
	elseif LASbook == 1 and self.HFocus1 == false then
		FocusWin:Show(false, false, 3)
    elseif LASbook == 2 and self.HFocus2 == true then
		FocusWin:Show(true, false, 3)
	elseif LASbook == 2 and self.HFocus2 == false then
		FocusWin:Show(false, false, 3)
    elseif LASbook == 3 and self.HFocus3 == true then
		FocusWin:Show(true, false, 3)
	elseif LASbook == 3 and self.HFocus3 == false then
		FocusWin:Show(false, false, 3)
    elseif LASbook == 4 and self.HFocus4 == true then
		FocusWin:Show(true, false, 3)
	elseif LASbook == 4 and self.HFocus4 == false then
		FocusWin:Show(false, false, 3)
	end
end

function Revolver:OnScaleChange(wndHandler, wndControl, fNewValue, fOldValue)
	self.scale = fNewValue
	self.wndForm:FindChild("Slider"):SetValue(self.scale)
	self.wndForm:FindChild("SliderLabel"):SetText(string.format("Scale (%.2f):", self.scale))
	self.wndMain:SetScale(self.scale)
end

function Revolver:OnButScaleChange(wndHandler, wndControl, fNewValue, fOldValue)
	self.butscale = fNewValue
	self.wndForm:FindChild("ButSlider"):SetValue(self.butscale)
	self.wndForm:FindChild("ButSliderLabel"):SetText(string.format("Scale (%.2f):", self.butscale))
	self.wndButtons:SetScale(self.butscale)
end

function Revolver:CHUD()
	if self.CHUD1 == true then
		Apollo.SetConsoleVariable("hud.skillsBarDisplay", 1)
	elseif self.CHUD1 == false then
		Apollo.SetConsoleVariable("hud.skillsBarDisplay", 2)
	end	
	if self.CHUD2 == true then
		Apollo.SetConsoleVariable("hud.resourceBarDisplay", 1)
	elseif self.CHUD2 == false then
		Apollo.SetConsoleVariable("hud.resourceBarDisplay", 2)
	end	
	Event_FireGenericEvent("OptionsUpdated_HUDPreferences")
end

function Revolver:ItemBarControl()
	if self.ItemBar == true then
		self.wndButtons:Show(true, false, 3)
	elseif self.ItemBar == false then
		self.wndButtons:Show(false, false, 3)
	end
end

function Revolver:ItemBarBGControl()
	if self.ItemBarBG == true then
		self.wndButtons:FindChild("BG"):Show(true, false, 3)
		self.wndButtons:FindChild("Gloss2"):Show(true, false, 3)
		self.wndButtons:FindChild("ButFrame"):Show(true, false, 3)
	elseif self.ItemBarBG == false then
		self.wndButtons:FindChild("BG"):Show(false, false, 3)
		self.wndButtons:FindChild("Gloss2"):Show(false, false, 3)
		self.wndButtons:FindChild("ButFrame"):Show(false, false, 3)
	end
end

-----------------------------------------------------------------------------------------------
-- Not a Spellslinger
-----------------------------------------------------------------------------------------------
function Revolver:NotSlinger()
end

-----------------------------------------------------------------------------------------------
-- The End!  :D
-----------------------------------------------------------------------------------------------
local RevolverInst = Revolver:new()
RevolverInst:Init()
